﻿using DotNetCore.CAP;
using Hotline.Application.FlowEngine;
using Hotline.FlowEngine;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.WorkflowModules;
using Hotline.FlowEngine.Workflows;
using Hotline.KnowledgeBase;
using Hotline.Orders;
using Hotline.Quality;
using Hotline.Repository.SqlSugar.CallCenter;
using Hotline.Repository.SqlSugar.Orders;
using Hotline.Settings;
using Hotline.Settings.TimeLimits;
using Hotline.Share.Dtos.FlowEngine.Workflow;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Dtos.Settings;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Quality;
using Hotline.Share.Enums.Settings;
using Hotline.Share.Mq;
using MapsterMapper;
using MediatR;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using XF.Domain.Entities;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Application.Handlers.FlowEngine;

public class NextStepHandler : INotificationHandler<NextStepNotify>
{
    private readonly IOrderDomainService _orderDomainService;
    private readonly IKnowledgeDomainService _knowledgeDomainService;
    private readonly IOrderRepository _orderRepository;
    private readonly ITimeLimitDomainService _timeLimitDomainService;
    private readonly IWorkflowDomainService _workflowDomainService;
    private readonly ICapPublisher _capPublisher;
    private readonly IMapper _mapper;
    private readonly ILogger<NextStepHandler> _logger;
    private readonly IRepository<OrderScreen> _orderScreenRepository;
    private readonly IQualityRepository _qualityRepository;

	public NextStepHandler(
        IOrderDomainService orderDomainService,
        IKnowledgeDomainService knowledgeDomainService,
        IOrderRepository orderRepository,
        ITimeLimitDomainService timeLimitDomainService,
        IWorkflowDomainService workflowDomainService,
        ICapPublisher capPublisher,
        IMapper mapper,
        ILogger<NextStepHandler> logger,
        IRepository<OrderScreen> orderScreenRepository,
        IQualityRepository qualityRepository)
    {
        _orderDomainService = orderDomainService;
        _knowledgeDomainService = knowledgeDomainService;
        _orderRepository = orderRepository;
        _timeLimitDomainService = timeLimitDomainService;
        _workflowDomainService = workflowDomainService;
        _capPublisher = capPublisher;
        _mapper = mapper;
        _logger = logger;
        _orderScreenRepository = orderScreenRepository;
        _qualityRepository = qualityRepository;
	}

    /// <summary>Handles a notification</summary>
    /// <param name="notification">The notification</param>
    /// <param name="cancellationToken">Cancellation token</param>
    public async Task Handle(NextStepNotify notification, CancellationToken cancellationToken)
    {
        _logger.LogInformation($"收到{nameof(NextStepNotify)}, notification: {JsonConvert.SerializeObject(notification)}");
        var workflow = notification.Workflow;
        var data = notification.Dto;

        DefinitionTag? currentTag = null;
        if (!string.IsNullOrEmpty(notification.Trace.Tag))
            currentTag = System.Text.Json.JsonSerializer.Deserialize<DefinitionTag>(notification.Trace.Tag);
        DefinitionTag? nextTag = null;
        if (!string.IsNullOrEmpty(notification.NextStepDefine.Tag))
            nextTag = System.Text.Json.JsonSerializer.Deserialize<DefinitionTag>(notification.NextStepDefine.Tag);


        switch (workflow.ModuleCode)
        {
            case WorkflowModuleConsts.OrderHandle:
                var order = await _orderDomainService.GetOrderAsync(workflow.ExternalId, cancellationToken);
                order.CheckIfFiled();
                _mapper.Map(workflow, order);

                var expiredTimeChanged = false;
                if (data.FlowDirection.HasValue && data.External.TimeLimit.HasValue && data.External.TimeLimitUnit.HasValue)
                {
                    // 1. calc expiredTime 2. update order.expiredTime 3. update workflow.expiredTime 4. publish province

                    var expiredTime = _timeLimitDomainService.CalcEndTime(DateTime.Now, data.External.TimeLimitUnit.Value,
                        data.External.TimeLimit.Value, data.FlowDirection is EFlowDirection.OrgToCenter);

                    if (data.FlowDirection is EFlowDirection.OrgToCenter)
                    {
                        order.OrgToCenter(expiredTime.EndTime);
                    }
                    else if (data.FlowDirection is EFlowDirection.CenterToOrg)
                    {
                        order.CenterToOrg(expiredTime.EndTime);
                        //写入质检
                        await _qualityRepository.AddQualityAsync(EQualitySource.Send, order.Id, cancellationToken);
					}

                    await _workflowDomainService.UpdateExpiredTimeAsync(workflow, expiredTime.EndTime,
                        expiredTime.RuleStr, data.External.TimeLimit, data.External.TimeLimitUnit, order.CenterToOrgTime, cancellationToken);

                    expiredTimeChanged = true;
                }
                await _orderRepository.UpdateAsync(order, cancellationToken);

                var orderDto = _mapper.Map<OrderDto>(order);
                await _capPublisher.PublishAsync(EventNames.HotlineOrderFlowHandled, new OrderFlowDto
                {
                    Order = orderDto,
                    WorkflowTrace = _mapper.Map<WorkflowTraceDto>(notification.Trace),
                    ExpiredTimeChanged = expiredTimeChanged,
                    HandlerOrgLevel = notification.HandlerOrgCode.CalcOrgLevel()
                }, cancellationToken: cancellationToken);

                break;
            case WorkflowModuleConsts.KnowledgeAdd:
            case WorkflowModuleConsts.KnowledgeUpdate:
            case WorkflowModuleConsts.KnowledgeDelete:
                break;
            case WorkflowModuleConsts.OrderScreen:
                var screen = await _orderScreenRepository.Queryable().Includes(x => x.Order)
                    .Where(x => x.Id == workflow.ExternalId).FirstAsync(cancellationToken);
                if (screen != null)
                {
                    screen.Status = EScreenStatus.Approval;
                    await _orderScreenRepository.UpdateAsync(screen, cancellationToken);
                }
                if (nextTag is not null)
                {
                    if (nextTag.Type == TagDefaults.TagType.Org)
                    {
                        switch (nextTag.Value)
                        {
                            case TagDefaults.TagValue.Province:
                                if (screen != null)
                                {
                                    var screenDto = _mapper.Map<OrderScreenListDto>(screen);
                                    if (screen.Order != null && screen.Order.Source == ESource.ProvinceStraight)
                                    {
                                        var screenOrderDto = _mapper.Map<OrderDto>(screen.Order);
                                        //推省上
                                        _capPublisher.Publish(EventNames.HotlineOrderScreenApply, new PublishScreenDto()
                                        {
                                            Order = screenOrderDto,
                                            Screen = screenDto,
                                            ClientGuid = ""
                                        });
                                    }
                                }
                                break;
                        }
                    }
                }
                break;
            case WorkflowModuleConsts.OrderDelay:
                if (nextTag is not null)
                {
                    if (nextTag.Type == TagDefaults.TagType.Org)
                    {
                        switch (nextTag.Value)
                        {
                            case TagDefaults.TagValue.Province:
                                //发起省延期审批
                                break;
                        }
                    }
                }
                break;
        }
    }
}