﻿using System.Text.Encodings.Web;
using System.Text.Json;
using System.Text.Unicode;
using DotNetCore.CAP;
using Hotline.CallCenter.Tels;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.WorkflowModules;
using Hotline.KnowledgeBase;
using Hotline.Orders;
using Hotline.Quality;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Quality;
using Hotline.Share.Mq;
using MapsterMapper;
using MediatR;
using Microsoft.Extensions.Logging;

namespace Hotline.Application.Handlers.FlowEngine
{
    public class StartWorkflowHandler : INotificationHandler<StartWorkflowNotify>
    {
        private readonly IOrderDomainService _orderDomainService;
        private readonly IKnowledgeDomainService _knowledgeDomainService;
        private readonly ITelDomainService _telDomainService;
        private readonly IOrderRepository _orderRepository;
        private readonly ICapPublisher _capPublisher;
        private readonly IMapper _mapper;
        private readonly ILogger<StartWorkflowHandler> _logger;
        private readonly IQualityRepository _qualityRepository;

		public StartWorkflowHandler(
            IOrderDomainService orderDomainService,
            IKnowledgeDomainService knowledgeDomainService,
            ITelDomainService telDomainService,
            IOrderRepository orderRepository,
            ICapPublisher capPublisher,
            IMapper mapper,
            ILogger<StartWorkflowHandler> logger,
            IQualityRepository qualityRepository)
        {
            _orderDomainService = orderDomainService;
            _knowledgeDomainService = knowledgeDomainService;
            _telDomainService = telDomainService;
            _orderRepository = orderRepository;
            _capPublisher = capPublisher;
            _mapper = mapper;
            _logger = logger;
            _qualityRepository = qualityRepository;

		}

        /// <summary>Handles a notification</summary>
        /// <param name="notification">The notification</param>
        /// <param name="cancellationToken">Cancellation token</param>
        public async Task Handle(StartWorkflowNotify notification, CancellationToken cancellationToken)
        {
            //基础拉丁字母和中日韩统一表意文字的基础Unicode 块(U+4E00-U+9FCC)。 基本涵盖了除使用西里尔字母以外所有西方国家的文字和亚洲中日韩越的文字
            _logger.LogInformation(
                $"收到{nameof(StartWorkflowNotify)}, notification: {JsonSerializer.Serialize(notification, new JsonSerializerOptions { Encoder = JavaScriptEncoder.Create(UnicodeRanges.BasicLatin, UnicodeRanges.CjkUnifiedIdeographs) })}");
            var workflow = notification.Workflow;
            var data = notification.Dto;

            switch (workflow.ModuleCode)
            {
                case WorkflowModuleConsts.OrderHandle:
                    var order = await _orderDomainService.GetOrderAsync(workflow.ExternalId, cancellationToken);
                    order.CheckIfFiled();
                    order.StartManageFlow();
                    _mapper.Map(workflow, order);
                    await _orderRepository.UpdateAsync(order, cancellationToken);
                    await _capPublisher.PublishAsync(EventNames.HotlineOrderFlowStarted, _mapper.Map<OrderDto>(order),
                        cancellationToken: cancellationToken);
                    //写入质检
                    await _qualityRepository.AddQualityAsync(EQualitySource.Accepted, order.Id, cancellationToken);
					break;
                case WorkflowModuleConsts.KnowledgeAdd:
                case WorkflowModuleConsts.KnowledgeUpdate:
                case WorkflowModuleConsts.KnowledgeDelete:
                    await _knowledgeDomainService.UpdateWorkFlowId(notification.FlowAssignInfo, workflow.ExternalId, workflow.Id, cancellationToken);
                    break;
                case WorkflowModuleConsts.TelRestApply:
                    await _telDomainService.TelRestFlowStartAsync(notification.FlowAssignInfo, workflow.ExternalId, workflow.Id, cancellationToken);
                    break;
            }
        }
    }
}
