﻿using Consul;
using Hotline.FlowEngine.Definitions;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Settings;
using SqlSugar;
using XF.Domain.Entities;
using XF.Domain.Repository;

namespace Hotline.FlowEngine.Workflows;

public abstract class StepBasicEntity : CreationEntity
{
    public string WorkflowId { get; set; }

    /// <summary>
    /// 流程指派类型
    /// </summary>
    public EFlowAssignType? FlowAssignType { get; set; }

    /// <summary>
    /// 该节点指派办理对象（依据不同指派方式可能为：orgId或userId）,该字段subStep才会存在，stepBox不存在
    /// 采用list类型，兼容或签
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true)]
    public List<Kv> Handlers { get; set; } = new();

    /// <summary>
    /// 上一节点办理时，nextStepCode下拉框中选中的值
    /// config模式：当前节点的difinition.code, dynamic模式：x级部门办理的x：int
    /// </summary>
    public string? PrevChosenStepCode { get; set; }

    #region 接办

    /// <summary>
    /// 接办人
    /// </summary>
    public string? AcceptorId { get; set; }

    public string? AcceptorName { get; set; }

    /// <summary>
    /// 接办人部门code
    /// </summary>
    public string? AcceptorOrgId { get; set; }

    public string? AcceptorOrgName { get; set; }

    /// <summary>
    /// 接办人部门行政区划代码
    /// </summary>
    public string? AcceptorOrgAreaCode { get; set; }

    /// <summary>
    /// 接办人部门行政区划名称
    /// </summary>
    public string? AcceptorOrgAreaName { get; set; }

    /// <summary>
    /// 接办时间
    /// </summary>
    public DateTime? AcceptTime { get; set; }

    #endregion

    #region 办理

    /// <summary>
    /// 办理人
    /// </summary>
    public string? HandlerId { get; set; }

    public string? HandlerName { get; set; }

    /// <summary>
    /// 办理人部门code
    /// </summary>
    public string? HandlerOrgId { get; set; }

    public bool? HandlerOrgIsCenter { get; set; }

    /// <summary>
    /// 办理人部门名称
    /// </summary>
    public string? HandlerOrgName { get; set; }

    /// <summary>
    /// 办理人部门行政区划代码
    /// </summary>
    public string? HandlerOrgAreaCode { get; set; }

    /// <summary>
    /// 办理人部门行政区划名称
    /// </summary>
    public string? HandlerOrgAreaName { get; set; }

    /// <summary>
    /// 办理完成时间
    /// </summary>
    public DateTime? HandleTime { get; set; }

    #endregion

    #region Definition

    public string Name { get; set; }

    /// <summary>
    /// 模板内唯一
    /// </summary>
    public string Code { get; set; }

    public EStepType StepType { get; init; }

    /// <summary>
    /// 节点业务类型
    /// </summary>
    public EBusinessType BusinessType { get; set; }

    /// <summary>
    /// 办理人类型
    /// </summary>
    public EHandlerType HandlerType { get; set; }

    /// <summary>
    /// 是否有否决按钮
    /// </summary>
    public bool CanReject { get; set; }

    /// <summary>
    /// 执行模式(自动与否)
    /// 只有普通节点支持自动，会签、动态节点均不支持自动
    /// </summary>
    public EExecuteMode ExecuteMode { get; set; }

    #region 会签相关配置

    /// <summary>
    /// 是否支持发起会签（即使支持发起，当下一节点为汇总或结束节点时亦不可发起）
    /// </summary>
    public bool CanStartCountersign { get; set; }

    /// <summary>
    /// 会签策略
    /// </summary>
    public EDynamicPolicy? CountersignPolicy { get; set; }

    #endregion

    #region 依据配置生成节点的方式

    /// <summary>
    /// 实例化模式
    /// </summary>
    public EInstanceMode InstanceMode { get; set; }

    /// <summary>
    /// 动态实例化策略(多次模式才有)
    /// </summary>
    public EDynamicPolicy? InstancePolicy { get; set; }

    /// <summary>
    /// 到此标记终止动态实例化(多次模式才有)
    /// <remarks>
    /// 按直属部门重复既保存orgLevel:int
    /// </remarks>
    /// </summary>
    public string? TerminalDynamicMark { get; set; }

    #endregion

    /// <summary>
    /// 标签
    /// </summary>
    public string? Tag { get; set; }

    #endregion

    #region 办理参数

    /// <summary>
    /// (下一节点办理人)根据审批者类型不同，此字段为不同内容
    /// <example>
    /// 部门等级/分类为：orgCodes, 角色为：userIds
    /// </example>
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true)]
    public List<Kv> NextHandlers { get; set; } = new();

    /// <summary>
    /// 下一节点主办，(NextHandlers其中一个, 如果不是会签则只有一个)
    /// </summary>
    public string? NextMainHandler { get; set; }

    /// <summary>
    /// 下一节点code（stepBox无值）
    /// </summary>
    public string? NextStepCode { get; set; }

    /// <summary>
    /// 是否回到会签发起节点汇总
    /// </summary>
    public bool BackToCountersignEnd { get; set; }

    public EFlowDirection? FlowDirection { get; set; }

    /// <summary>
    /// 是否短信通知
    /// </summary>
    public bool IsSms { get; set; }

    /// <summary>
    /// 办理意见
    /// </summary>
    [SugarColumn(Length = 2000)]
    public string? Opinion { get; set; }

    /// <summary>
    /// 附件
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true)]
    public List<string> Additions { get; set; } = new();

    /// <summary>
    /// 节点期满时间
    /// </summary>
    public DateTime? StepExpiredTime { get; set; }

    /// <summary>
    /// 发起会签
    /// </summary>
    public bool IsStartCountersign { get; set; }

    #endregion

    #region method

    public bool HasAccepted() => !string.IsNullOrEmpty(AcceptorId);

    public void Accept(
        string userId, string? userName,
        string orgId, string? orgName,
        string? orgAreaCode, string? orgAreaName)
    {
        AcceptorId = userId;
        AcceptorName = userName;
        AcceptorOrgId = orgId;
        AcceptorOrgName = orgName;
        AcceptorOrgAreaCode = orgAreaCode;
        AcceptorOrgAreaName = orgAreaName;
        AcceptTime = DateTime.Now;
    }

    public void Handle(
        string userId, string? userName,
        string orgId, string? orgName,
        string? orgAreaCode, string? orgAreaName,
        bool orgIsCenter, string? opinion)
    {
        if (!HasAccepted())
            Accept(userId, userName, orgId, orgName, orgAreaCode, orgAreaName);

        HandlerId = userId;
        HandlerName = userName;
        HandlerOrgId = orgId;
        HandlerOrgName = orgName;
        HandlerOrgAreaCode = orgAreaCode;
        HandlerOrgAreaName = orgAreaName;
        HandlerOrgIsCenter = orgIsCenter;
        HandleTime = DateTime.Now;

        if (!string.IsNullOrEmpty(opinion))
            Opinion = opinion;
    }

    #endregion
}